# -*- coding: utf-8 -*-
# Project : flacon
# File : configureDialog.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.



from settings import *

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui import *

import pyqticonloader


from configureDialog_ui import *
from flaconglobal import *



class ConfigureDialog(QtGui.QDialog):
    generalPageIndex  = 0
    flacPageIndex     = 1
    wvPageIndex       = 2
    oggPageIndex      = 3
    mp3PageIndex      = 4
    programsPageIndex = 5


    _dialog = None

   #****************************************************************
   #
   #****************************************************************
    @staticmethod
    def createAndShow(parent, pageIndex = -1):
        if not ConfigureDialog._dialog:
            ConfigureDialog._dialog = ConfigureDialog(parent)

        ConfigureDialog._dialog.show(pageIndex)
        ConfigureDialog._dialog.raise_()
        ConfigureDialog._dialog.activateWindow()


   #****************************************************************
   #
   #****************************************************************
    @staticmethod
    def pageIndex(audioFormat):
        if audioFormat == QVariant(outputAudioFormats.FLAC): return ConfigureDialog.flacPageIndex
        if audioFormat == QVariant(outputAudioFormats.WV):   return ConfigureDialog.wvPageIndex
        if audioFormat == QVariant(outputAudioFormats.OGG):  return ConfigureDialog.oggPageIndex
        if audioFormat == QVariant(outputAudioFormats.MP3):  return ConfigureDialog.mp3PageIndex

        return -1


   #****************************************************************
   #
   #****************************************************************
    def closeConfigureDialog(self):
        #if ConfigureDialog._configureDialog:
            ConfigureDialog._configureDialog = None

   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)

        self.form = Ui_configureDialog()
        self.form.setupUi(self)

        (width, ok)  = settings.value("ConfigureDialog/Width",  QVariant(self.size().width())).toInt()
        (height, ok) = settings.value("ConfigureDialog/Height", QVariant(self.size().height())).toInt()
        self.resize(width, height)

        self._programsEdits = []
        self._programsBtns  = []

        #***********************************************************
        self.connect(self.form.pages, SIGNAL('currentChanged(int)'), self.setPage)

        self.pagesListInit()

        self.generalInit()
        self.flacInit()
        self.wvInit()
        self.oggInit()
        self.mp3Init()
        self.programsInit()

        #***********************************************************
        self.generalLoad()
        self.flacLoad()
        self.wvLoad()
        self.oggLoad()
        self.mp3Load()
        self.programsLoad()

        self.connect(self.form.tmpDirButton, SIGNAL('clicked()'), self.tmpDirShowDialog)
        self.form.tmpDirButton.setIcon(pyqticonloader.icon(['document-open-folder', 'document-open']))


   #****************************************************************
   #
   #****************************************************************
    def done(self, res):
        settings.setValue("ConfigureDialog/Width",     QVariant(self.size().width()))
        settings.setValue("ConfigureDialog/Height",    QVariant(self.size().height()))

        if res:
            self.generalWrite()
            self.flacWrite()
            self.wvWrite()
            self.oggWrite()
            self.mp3Write()
            self.programsWrite()

        QDialog.done(self, res)
        self.setParent(None)
        ConfigureDialog._dialog = None


   #****************************************************************
   #
   #****************************************************************
    def show(self, pageIndex=-1):
        if pageIndex<0:
            pageIndex = self.form.pages.currentIndex()

        self.form.pagesList.setCurrentRow(pageIndex)
        QDialog.show(self)


   #****************************************************************
   #
   #****************************************************************
    def setPage(self, pageIndex):
        self.form.pagesList.setCurrentRow(pageIndex)


   #****************************************************************
   #
   #****************************************************************
    def pagesListInit(self):
        pl = self.form.pagesList
        pl.item(ConfigureDialog.generalPageIndex).setIcon(pyqticonloader.icon('go-home'))
        pl.item(ConfigureDialog.flacPageIndex).setIcon(pyqticonloader.icon(['audio-x-flac', 'audio-x-generic']))
        pl.item(ConfigureDialog.wvPageIndex).setIcon(pyqticonloader.icon(['audio-x-wv', 'audio-x-generic']))
        pl.item(ConfigureDialog.oggPageIndex).setIcon(pyqticonloader.icon(['audio-x-ogg', 'audio-x-generic']))
        pl.item(ConfigureDialog.mp3PageIndex).setIcon(pyqticonloader.icon(['audio-x-mp3', 'audio-x-generic']))
        pl.item(ConfigureDialog.programsPageIndex).setIcon(pyqticonloader.icon('applications-system'))

        width = 0
        for i in xrange(0, pl.count()):
            item = pl.item(i)
            width = max(width, pl.fontMetrics().width(item.text()))

        width = width + 42

        for i in xrange(0, pl.count()):
            item = pl.item(i)
            h =  pl.iconSize().height() + pl.fontMetrics().height() + 8
            item.setSizeHint(QSize(width, h))


        pl.setMaximumWidth(width + 2 * pl.frameWidth())


   #****************************************************************
   #
   #****************************************************************
    def generalInit(self):
        pass


   #****************************************************************
   #
   #****************************************************************
    def generalLoad(self):
        self.loadSpinBox('Encoder/ThreadCount', self.form.threadsCountSpin)
        self.loadLineEdit('Encoder/TmpDir', self.form.tmpDirEdit)


   #****************************************************************
   #
   #****************************************************************
    def generalWrite(self):
        self.writeSpinBox('Encoder/ThreadCount', self.form.threadsCountSpin)
        self.writeLineEdit('Encoder/TmpDir', self.form.tmpDirEdit)


   #****************************************************************
   #
   #****************************************************************
    def flacInit(self):
        self.form.flacCompressionSpin.setToolTip(self.form.flacCompressionSlider.toolTip())
        self.fillReplayGainComboBox(self.form.flacGainCbx)


   #****************************************************************
   #
   #****************************************************************
    def flacLoad(self):
        self.loadSlider('Flac/Compression', self.form.flacCompressionSlider)
        self.loadComboBox('Flac/ReplayGain',  self.form.flacGainCbx)


   #****************************************************************
   #
   #****************************************************************
    def flacWrite(self):
        self.writeSlider('Flac/Compression', self.form.flacCompressionSlider)
        self.writeComboBox('Flac/ReplayGain',   self.form.flacGainCbx)


   #****************************************************************
   #
   #****************************************************************
    def wvInit(self):
        self.form.wvCompressionSpin.setToolTip(self.form.wvCompressionSlider.toolTip())
        self.fillReplayGainComboBox(self.form.wvGainCbx)


   #****************************************************************
   #
   #****************************************************************
    def wvLoad(self):
        self.loadSlider('WV/Compression', self.form.wvCompressionSlider)
        self.loadComboBox('WV/ReplayGain',  self.form.wvGainCbx)


   #****************************************************************
   #
   #****************************************************************
    def wvWrite(self):
        self.writeSlider('WV/Compression', self.form.wvCompressionSlider)
        self.writeComboBox('WV/ReplayGain',   self.form.wvGainCbx)



   #****************************************************************
   #
   #****************************************************************
    def oggInit(self):
        self.form.oggQualitySpin.setToolTip(self.form.oggQualitySlider.toolTip())
        self.form.oggQualityLabel.setToolTip(self.form.oggQualitySlider.toolTip())
        self.connect(self.form.oggQualitySlider, SIGNAL('valueChanged(int)'), self.oggQualitySliderChanged)
        self.connect(self.form.oggQualitySpin, SIGNAL('valueChanged(double)'), self.oggQualitySpinChanged)
        self.fillReplayGainComboBox(self.form.oggGainCbx)
        self.form.oggGainCbx.setToolTip(self.form.flacGainCbx.toolTip())


   #****************************************************************
   #
   #****************************************************************
    def oggLoad(self):
        self.form.oggUseQualityCheck.setChecked(settings.value('Ogg/UseQuality').toBool())
        self.loadSpinBox('Ogg/Quality', self.form.oggQualitySpin)
        self.loadComboBox('Ogg/MinBitrate',  self.form.oggMinBitrateCbx)
        self.loadComboBox('Ogg/NormBitrate', self.form.oggNormBitrateCbx)
        self.loadComboBox('Ogg/MaxBitrate',  self.form.oggMaxBitrateCbx)
        self.loadComboBox('Ogg/ReplayGain',  self.form.oggGainCbx)


   #****************************************************************
   #
   #****************************************************************
    def oggWrite(self):
        settings.setValue('Ogg/UseQuality', self.form.oggUseQualityCheck.isChecked())
        self.writeSpinBox('Ogg/Quality', self.form.oggQualitySpin)
        self.writeComboBox('Ogg/MinBitrate',  self.form.oggMinBitrateCbx)
        self.writeComboBox('Ogg/NormBitrate', self.form.oggNormBitrateCbx)
        self.writeComboBox('Ogg/MaxBitrate',  self.form.oggMaxBitrateCbx)
        self.writeComboBox('Ogg/ReplayGain',   self.form.oggGainCbx)


   #****************************************************************
   #
   #****************************************************************
    def mp3Init(self):
        presets = self.form.mp3PresetCbx

        presets.addItem(self.tr('AVR medium'),        QVariant('avrMedium'))
        presets.addItem(self.tr('AVR standard'),      QVariant('avrStandard'))
        presets.addItem(self.tr('AVR standard fast'), QVariant('avrStandardFast'))
        presets.addItem(self.tr('AVR extreme'),       QVariant('avrExtreme'))
        presets.addItem(self.tr('AVR extreme fast'),  QVariant('avrExtremeFast'))
        presets.addItem(self.tr('CBR insane'),        QVariant('cbrInsane'))
        presets.addItem(self.tr('CBR kbps'),          QVariant('cbrKbps'))
        presets.addItem(self.tr('ABR kbps'),          QVariant('abrKbps'))
        presets.addItem(self.tr('VBR quality'),       QVariant('vbrQuality'))


        bitrates = self.form.mp3BitrateCbx
        bitrates.addItem(self.tr('32 kbps'),  QVariant(32))
        bitrates.addItem(self.tr('40 kbps'),  QVariant(40))
        bitrates.addItem(self.tr('48 kbps'),  QVariant(48))
        bitrates.addItem(self.tr('56 kbps'),  QVariant(56))
        bitrates.addItem(self.tr('64 kbps'),  QVariant(64))
        bitrates.addItem(self.tr('80 kbps'),  QVariant(80))
        bitrates.addItem(self.tr('96 kbps'),  QVariant(96))
        bitrates.addItem(self.tr('112 kbps'), QVariant(112))
        bitrates.addItem(self.tr('128 kbps'), QVariant(128))
        bitrates.addItem(self.tr('160 kbps'), QVariant(160))
        bitrates.addItem(self.tr('192 kbps'), QVariant(192))
        bitrates.addItem(self.tr('224 kbps'), QVariant(224))
        bitrates.addItem(self.tr('256 kbps'), QVariant(256))
        bitrates.addItem(self.tr('320 kbps'), QVariant(320))

        self.form.mp3QualitySpin.setToolTip(self.form.mp3QualitySlider.toolTip())
        self.form.mp3QualityLabel.setToolTip(self.form.mp3QualitySlider.toolTip())

        self.connect(self.form.mp3PresetCbx, SIGNAL('currentIndexChanged(int)'), self.mp3PresetCbxCanged)
        self.mp3PresetCbxCanged(self.form.mp3PresetCbx.currentIndex())

        self.fillReplayGainComboBox(self.form.mp3GainCbx)
        self.form.mp3GainCbx.setToolTip(self.form.flacGainCbx.toolTip())


   #****************************************************************
   #
   #****************************************************************
    def mp3Load(self):
        self.loadComboBox('Mp3/Preset',      self.form.mp3PresetCbx)
        self.loadComboBox('Mp3/Bitrate',     self.form.mp3BitrateCbx)
        self.loadSpinBox('Mp3/Quality',      self.form.mp3QualitySpin)
        self.loadComboBox('Mp3/ReplayGain',  self.form.mp3GainCbx)


   #****************************************************************
   #
   #****************************************************************
    def mp3Write(self):
        self.writeComboBox('Mp3/Preset',  self.form.mp3PresetCbx)
        self.writeComboBox('Mp3/Bitrate', self.form.mp3BitrateCbx)
        self.writeSpinBox('Mp3/Quality',  self.form.mp3QualitySpin)
        self.writeComboBox('Mp3/ReplayGain',   self.form.mp3GainCbx)


   #****************************************************************
   #
   #****************************************************************
    def programsInit(self):

        row=0
        for program in settings.programs():
            edit = QLineEdit(self.form.programsPage)
            edit.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

            label = QLabel(self.form.programsPage)
            label.setText(QString(program + ':'))
            label.setBuddy(edit)

            btn = QToolButton(self.form.programsPage)
            btn.setIcon(pyqticonloader.icon(['document-open-folder', 'document-open']))
            self.connect(btn, SIGNAL('clicked()'), self.openProgFile)

            self.form.progsLayout.addWidget(label, row, 0)
            self.form.progsLayout.addWidget(edit,  row, 1)
            self.form.progsLayout.addWidget(btn,   row, 2)
            row+=1

            self._programsEdits.append(edit)
            self._programsBtns.append(btn)

        self.connect(self.form.progScanButton, SIGNAL('clicked()'), self.findPrograms)


   #****************************************************************
   #
   #****************************************************************
    def programsLoad(self):
        n=0
        for program in settings.programs():
            self.loadLineEdit("Programs/"+program, self._programsEdits[n])
            n+=1


   #****************************************************************
   #
   #****************************************************************
    def programsWrite(self):
        n=0
        for program in settings.programs():
            self.writeLineEdit("Programs/"+program, self._programsEdits[n])
            n+=1


   #****************************************************************
   #
   #****************************************************************
    def oggQualitySliderChanged(self, value):
        val = value / 10.0
        self.form.oggQualitySpin.setValue(val)


   #****************************************************************
   #
   #****************************************************************
    def oggQualitySpinChanged(self, value):
        self.form.oggQualitySlider.setValue(value * 10)
        #       -1  0   1   2   3    4    5    6    7    8    9    10   fake
        kbps = [45, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 500, 500]
        x1 = int(value)
        y1=kbps[x1+1];
        y2=kbps[x1+2]
        bitrate = round((y2-y1+0.0) * (value - x1) + y1)
        self.form.oggQualityLabel.setText(QString("(~%1 kbps)").arg(bitrate))

        self.fillOggBitrateComboBox(self.form.oggMinBitrateCbx)
        self.fillOggBitrateComboBox(self.form.oggNormBitrateCbx)
        self.fillOggBitrateComboBox(self.form.oggMaxBitrateCbx)


   #****************************************************************
   #
   #****************************************************************
    def fillOggBitrateComboBox(self, comboBox):
        comboBox.addItem(self.tr('Default'),  QVariant(''))
        comboBox.addItem(self.tr('64 kbps'),  QVariant('64'))
        comboBox.addItem(self.tr('80 kbps'),  QVariant('80'))
        comboBox.addItem(self.tr('96 kbps'),  QVariant('96'))
        comboBox.addItem(self.tr('128 kbps'), QVariant('128'))
        comboBox.addItem(self.tr('160 kbps'), QVariant('160'))
        comboBox.addItem(self.tr('196 kbps'), QVariant('196'))
        comboBox.addItem(self.tr('256 kbps'), QVariant('256'))
        comboBox.addItem(self.tr('350 kbps'), QVariant('350'))


   #****************************************************************
   #
   #****************************************************************
    def fillReplayGainComboBox(self, comboBox):
        comboBox.clear()
        comboBox.addItem(self.tr('Disabled'),  QVariant(ReplayGainTypes.Disable))
        comboBox.addItem(self.tr('Per Track'), QVariant(ReplayGainTypes.Track))
        comboBox.addItem(self.tr('Per Album'), QVariant(ReplayGainTypes.Album))


   #****************************************************************
   #
   #****************************************************************
    def mp3PresetCbxCanged(self, index):
        preset = self.form.mp3PresetCbx.itemData(index).toString()

        enable = (preset == 'abrKbps' or preset=='cbrKbps')
        self.form.mp3BitrateBox.setEnabled(enable)

        enable = (preset == 'vbrQuality')
        self.form.mp3QualityBox.setEnabled(enable)


   #****************************************************************
   #
   #****************************************************************
    def loadComboBox(self, key, comboBox):
        data = settings.value(key)
        comboBox.setCurrentIndex(comboBox.findData(data))


   #****************************************************************
   #
   #****************************************************************
    def writeComboBox(self, key, comboBox):
        data = comboBox.itemData(comboBox.currentIndex())
        settings.setValue(key, data)


   #****************************************************************
   #
   #****************************************************************
    def loadSpinBox(self, key, spinBox):
        if isinstance(spinBox, QDoubleSpinBox):
            (value, ok) = settings.value(key).toDouble()
            if ok:  spinBox.setValue(value)

        if isinstance(spinBox, QSpinBox):
            (value, ok) = settings.value(key).toInt()
            if ok:  spinBox.setValue(value)


   #****************************************************************
   #
   #****************************************************************
    def writeSpinBox(self, key, spinBox):
         settings.setValue(key, spinBox.value())


   #****************************************************************
   #
   #****************************************************************
    def loadSlider(self, key, slider):
        (value, ok) = settings.value(key).toInt()
        if ok:
            slider.setValue(value)


   #****************************************************************
   #
   #****************************************************************
    def writeSlider(self, key, slider):
        settings.setValue(key, slider.value())


   #****************************************************************
   #
   #****************************************************************
    def loadLineEdit(self, key, edit):
        edit.setText(settings.value(key).toString())


   #****************************************************************
   #
   #****************************************************************
    def writeLineEdit(self, key, edit):
        settings.setValue(key, QVariant(edit.text()))


   #****************************************************************
   #
   #****************************************************************
    def openProgFile(self):
        n = self._programsBtns.index(self.sender())
        edit = self._programsEdits[n]
        progName = settings.programs[n]

        flt = self.tr("%1 program").arg(progName)  + QString(" (%1*);;").arg(progName) + \
              self.tr("All files")              + " (*.*)"

        fileName = QFileDialog.getOpenFileName(self, self.tr("Select program file"), "/usr/bin/", flt)
        if (fileName):
            edit.setText(progFile)


   #****************************************************************
   #
   #****************************************************************
    def findPrograms(self):
        self.programsWrite()
        settings.findPrograms()
        self.programsLoad()


   #****************************************************************
   #
   #****************************************************************
    def tmpDirShowDialog(self):
        tmpDir = QFileDialog.getExistingDirectory(self, self.tr("Select temporary directory"), self.form.tmpDirEdit.text())
        if (tmpDir):
            self.form.tmpDirEdit.setText(tmpDir)
