# -*- coding: utf-8 -*-
# Project : flacon
# File : settings.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
import subprocess
import flaconglobal


class Settings(QSettings):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, organization, application):
        QSettings.__init__(self, organization, application)

        self._programs = (
            'shntool',
            'flac',
            'mac',
            'oggenc',
            'lame',
            'wavpack',
            'metaflac',
            'vorbisgain',
            'mp3gain',
            'ttaenc'
        )

        defaults = {
            'Tags/Codepage':        'UTF-8',

            # Globals *************************************************
            'Encoder/ThreadCount':  8,
            'Encoder/TmpDir':       '',

            # Out Files ***********************************************
            'OutFiles/Pattern':     '%a/%y - %A/%n - %t',
            'OutFiles/Directory':   '~/music',
            'OutFiles/Format':      'FLAC',

            # FLAC *****************************************************
            'Flac/Compression':     5,
            'Flac/ReplayGain':      flaconglobal.ReplayGainTypes.Disable,

            # OGG ******************************************************
            'Ogg/UseQuality':       True,
            'Ogg/Quality':          7,
            'Ogg/MinBitrate':       '',
            'Ogg/NormBitrate':      '',
            'Ogg/MaxBitrate':       '',
            'Ogg/ReplayGain':       flaconglobal.ReplayGainTypes.Disable,

            # MP3 ******************************************************
            'Mp3/Preset':           'avrStandard',
            'Mp3/Bitrate':          '320',
            'Mp3/Quality':          4,
            'Mp3/ReplayGain':       flaconglobal.ReplayGainTypes.Disable,

            # WavPack **************************************************
            'WV/Compression':       1,
            'WV/ReplayGain':        flaconglobal.ReplayGainTypes.Disable,

            # Internet *************************************************
            'Inet/CDDBHost':        'freedb.freedb.org',
        }

        self._defaults = {}
        for key, val in defaults.iteritems():
            self._defaults[QString(key)] = QVariant(val)

        self.findPrograms()


   #****************************************************************
   #
   #****************************************************************
    def programs(self):
        return self._programs


   #****************************************************************
   #
   #****************************************************************
    def value(self, key, defaultValue = None):
        key = QString(key)
        if defaultValue == None:
            try:
                defaultValue = self._defaults[key]
            except KeyError:
                defaultValue = QVariant()

        return QSettings.value(self, key, QVariant(defaultValue))


   #****************************************************************
   # Sets the value of setting key to value.
   #****************************************************************
    def setValue(self, key, value):
        QSettings.setValue(self, QString(key), QVariant(value))
        self.emit(SIGNAL("changed()"))


   #****************************************************************
   #
   #****************************************************************
    def findPrograms(self):
        for program in self._programs:
            if (self.value('Programs/'+program).toString().isEmpty()):
                path = subprocess.Popen(["which", program], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0]
                self.setValue('Programs/'+program, QString(path).trimmed())


   #****************************************************************
   #
   #****************************************************************
    def checkProgram(self, programKey):
        val = self.value(programKey).toString()
        if val.isEmpty():
            return False

        return QFileInfo(val).exists()


   #****************************************************************
   #
   #****************************************************************
    def event(self, event):
        res = QSettings.event(self, event)
        if event.type() == QEvent.UpdateRequest:
            self.emit(SIGNAL("Changed()"))

        return res

settings = Settings('flacon', 'flacon')
